<?php

namespace FiloBlu\ExtViSualMerchandiser\Helper;

use Exception;
use FiloBlu\ExtViSualMerchandiser\Model\StockStatus;
use FiloBlu\ExtViSualMerchandiser\Model\StockStatusInterface;
use FiloBlu\ExtViSualMerchandiser\Model\StockStatusInterfaceFactory;
use Magento\CatalogInventory\Api\StockRegistryInterface;
use Magento\CatalogInventory\Model\Stock;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Module\Manager;
use Magento\Framework\ObjectManagerInterface;

/**
 * Class StockHelper
 * @package FiloBlu\ExtViSualMerchandiser\Helper
 */
class StockHelper extends AbstractHelper
{

    /**
     * @var StockStatusInterface []
     */
    protected $stockStatus = [];
    /**
     * @var Manager
     */
    private $moduleManager;
    /**
     * @var ObjectManagerInterface
     */
    private $objectManager;
    /**
     * @var StockStatusInterfaceFactory
     */
    private $stockStatusFactory;
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;
    /**
     * @var ConfigHelper
     */
    private $configHelper;

    /**
     * @var
     */
    protected $connection;

    /**
     * StockHelper constructor.
     * @param Context $context
     * @param Manager $moduleManager
     * @param StockStatusInterfaceFactory $stockStatusFactory
     * @param ResourceConnection $resourceConnection
     * @param ConfigHelper $configHelper
     * @param ObjectManagerInterface $objectManager
     */
    public function __construct(
        Context $context,
        Manager $moduleManager,
        StockStatusInterfaceFactory $stockStatusFactory,
        ResourceConnection $resourceConnection,
        ConfigHelper $configHelper,
        ObjectManagerInterface $objectManager

    )
    {
        parent::__construct($context);
        $this->moduleManager = $moduleManager;
        $this->objectManager = $objectManager;
        $this->stockStatusFactory = $stockStatusFactory;
        $this->resourceConnection = $resourceConnection;
        $this->configHelper = $configHelper;
    }

    /**
     * @param $sku
     * @return StockStatusInterface[]
     */
    public function getStockStatus(\Magento\Catalog\Api\Data\ProductInterface $product)
    {
        $sku = $product->getSku();

        if ($this->configHelper->isMultistockAvailable()) {

            $result = [];

            foreach ($this->getMsiStocks() as $stockId => $stockStatus) {
                try {
                    $this->stockStatus[$stockId]->setIsInStock($sku, $this->getMsiQuantity($product, $stockId));
                    $result[$stockId] = $this->stockStatus[$stockId]->isInStock($sku);
                } catch (Exception $e) {
                    continue;
                }
            }

            return $result;
        }

        if (empty($this->stockStatus)) {
            $this->stockStatus[Stock::DEFAULT_STOCK_ID] = $this->objectManager->get(StockRegistryInterface::class);
        }

        if ($product->getTypeId() === \Magento\ConfigurableProduct\Model\Product\Type\Configurable::TYPE_CODE) {
            $children = $product->getTypeInstance()->getUsedProducts($product);

            $qty = 0;
            foreach ($children as $child) {
                $childIsInStock = $this->objectManager->create('\Magento\CatalogInventory\Model\Stock\Item')->load($child->getId(), 'product_id');
                $qty += $childIsInStock->getQty();
            }
            $product->setData("stock", $qty);
            return [Stock::DEFAULT_STOCK_ID => $qty > 0? Stock::STOCK_IN_STOCK : Stock::STOCK_OUT_OF_STOCK];
        }

        /** @var StockRegistryInterface $stockStatus */
        $stockStatus = $this->stockStatus[Stock::DEFAULT_STOCK_ID];
        return [Stock::DEFAULT_STOCK_ID => $stockStatus->getStockStatusBySku($sku)->getStockStatus() === Stock::STOCK_IN_STOCK];
    }

    /**
     * @return StockStatusInterface[]
     */
    protected function getMsiStocks()
    {
        if (!empty($this->stockStatus)) {
            return $this->stockStatus;
        }

        $default = $this->configHelper->getSelectedDefaultStockId();
        $stocks = $this->configHelper->getAvailableStocks();

        foreach ($stocks as $stockId => $name) {
            /** @var StockStatusInterface $stockStatus */
            $stockStatus = $this->stockStatusFactory->create();
            $stockStatus->setId($stockId);
            $stockStatus->setName($name);
            $this->stockStatus[$stockId] = $stockStatus;
        }

        uksort($this->stockStatus, function ($a, $b) use ($default) {
            return ($a == $default || $b == $default);
        });
        return $this->stockStatus;
    }

    /**
     * @param \Magento\Catalog\Api\Data\ProductInterface $product
     * @param int $stockId
     * @return mixed
     */
    public function getMsiQuantity(\Magento\Catalog\Api\Data\ProductInterface $product, $stockId)
    {
        $qty = 0;
        $connection = $this->getConnection();
        $inventoryStock = $connection->getTableName("inventory_stock_{$stockId}");
        $reservation = $connection->getTableName('inventory_reservation');

        $skus = [];

        if($product->getTypeId() === \Magento\ConfigurableProduct\Model\Product\Type\Configurable::TYPE_CODE)
        {
            $children = $product->getTypeInstance()->getUsedProducts($product);
            foreach ($children as $child) {
                $skus[] = '\'' . $child->getSku() . '\'';
            }
        }else{
            $skus[] = '\'' . $product->getSku() . '\'';
        }

        $joinedSku = implode(',', $skus);
        $qty1 = (float)$connection->fetchOne("SELECT IFNULL(SUM(`quantity`), 0) FROM `{$inventoryStock}` WHERE `sku` IN ($joinedSku)");
        $qty2 = (float)$connection->fetchOne("SELECT  IFNULL(SUM(`quantity`), 0) FROM `{$reservation}` WHERE  `stock_id` = $stockId AND sku IN ($joinedSku)");

        $qty = $qty1 + $qty2;

        if($stockId == 1){
            $product->setData('stock', $product->getData('stock') + $qty);
        }
        else {
            $product->setData('stock', $qty);
        }

        return $qty > 0;
    }

    protected function getConnection()
    {
        if($this->connection) {
            return $this->connection;
        }

        return ($this->connection = $this->resourceConnection->getConnection());
    }
    /**
     * @param $stockId
     * @return string
     */
    public function getStockLabel($stockId)
    {
        if ((int)$stockId === Stock::DEFAULT_STOCK_ID) {
            return 'Default';
        }

        if (isset($this->stockStatus[$stockId])) {
            return $this->stockStatus[$stockId]->getName();
        }

        return null;
    }
}
